@@ -1,9 +1,9 @@
 package Net::DNS::RR;
 
-#
-# $Id: RR.pm 1696 2018-07-20 16:15:11Z willem $
-#
-our $VERSION = (qw$LastChangedRevision: 1696 $)[1];
+use strict;
+use warnings;
+
+our $VERSION = (qw$Id: RR.pm 1847 2021-08-11 10:02:44Z willem $)[2];
 
 
 =head1 NAME
@@ -14,12 +14,12 @@ Net::DNS::RR - DNS resource record base class
 
     use Net::DNS;
 
-    $rr = new Net::DNS::RR('example.com IN A 192.0.2.99');
+    $rr = Net::DNS::RR->new('example.com IN AAAA 2001:DB8::1');
 
-    $rr = new Net::DNS::RR(
+    $rr = Net::DNS::RR->new(
 	    owner   => 'example.com',
-	    type    => 'A',
-	    address => '192.0.2.99'
+	    type    => 'AAAA',
+	    address => '2001:DB8::1'
 	    );
 
 
@@ -31,15 +31,12 @@ See also the manual pages for each specific RR type.
 =cut
 
 
-use strict;
-use warnings;
 use integer;
 use Carp;
 
-use constant LIB => grep $_ ne '.', grep !ref($_), @INC;
+use constant LIB => grep { $_ ne '.' } grep { !ref($_) } @INC;
 
-use Net::DNS::Parameters;
-use Net::DNS::Domain;
+use Net::DNS::Parameters qw(%classbyname :class :type);
 use Net::DNS::DomainName;
 
 
@@ -58,8 +55,8 @@ sub new {
 		scalar @_ > 2 ? &_new_hash : &_new_string;
 	} || do {
 		my $class = shift || __PACKAGE__;
-		my @param = map defined($_) ? split /\s+/ : 'undef', @_;
-		my $stmnt = substr "new $class( @param )", 0, 80;
+		my @param = map { defined($_) ? split /\s+/ : 'undef' } @_;
+		my $stmnt = substr "$class->new( @param )", 0, 80;
 		croak "${@}in $stmnt\n";
 	};
 }
@@ -67,10 +64,10 @@ sub new {
 
 =head2 new (from string)
 
-    $a	   = new Net::DNS::RR('host.example.com. 86400 A 192.0.2.1');
-    $mx	   = new Net::DNS::RR('example.com. 7200 MX 10 mailhost.example.com.');
-    $cname = new Net::DNS::RR('www.example.com 300 IN CNAME host.example.com');
-    $txt   = new Net::DNS::RR('txt.example.com 3600 HS TXT "text data"');
+    $aaaa  = Net::DNS::RR->new('host.example.com. 86400 AAAA 2001:DB8::1');
+    $mx	   = Net::DNS::RR->new('example.com. 7200 MX 10 mailhost.example.com.');
+    $cname = Net::DNS::RR->new('www.example.com 300 IN CNAME host.example.com');
+    $txt   = Net::DNS::RR->new('txt.example.com 3600 HS TXT "text data"');
 
 Returns an object of the appropriate RR type, or a L<Net::DNS::RR> object
 if the type is not implemented. The attribute values are extracted from the
@@ -88,7 +85,7 @@ The trailing dot (.) is optional.
 
 =cut
 
-my $PARSE_REGEX = q/("[^"]*")|;[^\n]*|[ \t\n\r\f()]/;
+my $PARSE_REGEX = q/("[^"]*")|;[^\n]*|[ \t\n\r\f()]+/;		# NB: *not* \s (matches Unicode white space)
 
 sub _new_string {
 	my $base;
@@ -103,21 +100,21 @@ sub _new_string {
 	s/\\\(/\\040/g;						# disguise escaped bracket
 	s/\\\)/\\041/g;						# disguise escaped bracket
 	s/\\;/\\059/g;						# disguise escaped semicolon
-	my ( $owner, @token ) = grep defined && length, split /$PARSE_REGEX/o;
+	my ( $owner, @token ) = grep { defined && length } split /$PARSE_REGEX/o;
 
 	croak 'unable to parse RR string' unless scalar @token;
 	my $t1 = uc $token[0];
-	my $t2 = uc $token[1] if $#token;
+	my $t2 = $token[1];
 
 	my ( $ttl, $class );
 	if ( not defined $t2 ) {				# <owner> <type>
 		@token = ('ANY') if $classbyname{$t1};		# <owner> <class>
+	} elsif ( $t1 =~ /^\d/ ) {
+		$ttl   = shift @token;				# <owner> <ttl> [<class>] <type>
+		$class = shift @token if $classbyname{uc $t2} || $t2 =~ /^CLASS\d/i;
 	} elsif ( $classbyname{$t1} || $t1 =~ /^CLASS\d/ ) {
 		$class = shift @token;				# <owner> <class> [<ttl>] <type>
-		$ttl = shift @token if $t2 =~ /^\d/;
-	} elsif ( $t1 =~ /^\d/ ) {
-		$ttl = shift @token;				# <owner> <ttl> [<class>] <type>
-		$class = shift @token if $classbyname{$t2} || $t2 =~ /^CLASS\d/;
+		$ttl   = shift @token if $t2 =~ /^\d/;
 	}
 
 	my $type      = shift(@token);
@@ -136,27 +133,28 @@ sub _new_string {
 		my $rdata = pack 'H*', join( '', @token );
 		croak 'length and hexadecimal data inconsistent' unless $rdlen == length $rdata;
 		$self->rdata($rdata);				# unpack RDATA
-		return $self;
+	} else {
+		$self->_parse_rdata(@token);			# parse arguments
 	}
 
-	$self->_parse_rdata(@token);				# parse arguments
+	$self->_post_parse();
 	return $self;
 }
 
 
 =head2 new (from hash)
 
-    $rr = new Net::DNS::RR(%hash);
+    $rr = Net::DNS::RR->new(%hash);
 
-    $rr = new Net::DNS::RR(
+    $rr = Net::DNS::RR->new(
 	    owner   => 'host.example.com',
 	    ttl	    => 86400,
 	    class   => 'IN',
-	    type    => 'A',
-	    address => '192.0.2.1'
+	    type    => 'AAAA',
+	    address => '2001:DB8::1'
 	    );
  
-    $rr = new Net::DNS::RR(
+    $rr = Net::DNS::RR->new(
 	    owner   => 'txt.example.com',
 	    type    => 'TXT',
 	    txtdata => [ 'one', 'two' ]
@@ -196,6 +194,7 @@ sub _new_hash {
 	};
 	die ref($self) eq __PACKAGE__ ? "type $type not implemented" : () if $@;
 
+	$self->_post_parse();
 	return $self;
 }
 
@@ -270,7 +269,7 @@ sub encode {
 	my $owner = $self->{owner}->encode( $offset, @opaque );
 	my ( $type, $class, $ttl ) = @{$self}{qw(type class ttl)};
 	my $rdata = $self->_empty ? '' : $self->_encode_rdata( $offset + length($owner) + RRFIXEDSZ, @opaque );
-	pack 'a* n2 N n a*', $owner, $type, $class || 1, $ttl || 0, length $rdata, $rdata;
+	return pack 'a* n2 N n a*', $owner, $type, $class || 1, $ttl || 0, length $rdata, $rdata;
 }
 
 
@@ -293,7 +292,7 @@ sub canonical {
 	my $owner = $self->{owner}->canonical;
 	my ( $type, $class, $ttl ) = @{$self}{qw(type class ttl)};
 	my $rdata = $self->_empty ? '' : $self->_encode_rdata( length($owner) + RRFIXEDSZ );
-	pack 'a* n2 N n a*', $owner, $type, $class || 1, $ttl || 0, length $rdata, $rdata;
+	return pack 'a* n2 N n a*', $owner, $type, $class || 1, $ttl || 0, length $rdata, $rdata;
 }
 
 
@@ -301,13 +300,14 @@ sub canonical {
 
     $rr->print;
 
-Prints the record to the standard output.  Calls the string method
-to get the formatted RR representation.
+Prints the resource record to the currently selected output filehandle.
+Calls the string method to get the formatted RR representation.
 
 =cut
 
 sub print {
 	print shift->string, "\n";
+	return;
 }
 
 
@@ -315,10 +315,10 @@ sub print {
 
     print $rr->string, "\n";
 
-Returns a string representation of the RR using the zone file format
-described in RFC1035.  All domain names are fully qualified with
-trailing dot.  This differs from RR attribute methods, which omit
-the trailing dot.
+Returns a string representation of the RR using the master file format
+mandated by RFC1035.
+All domain names are fully qualified with trailing dot.
+This differs from RR attribute methods, which omit the trailing dot.
 
 =cut
 
@@ -326,7 +326,7 @@ sub string {
 	my $self = shift;
 
 	my $name = $self->{owner}->string;
-	my @ttl	 = grep defined, $self->{ttl};
+	my @ttl	 = grep {defined} $self->{ttl};
 	my @core = ( $name, @ttl, $self->class, $self->type );
 
 	my $empty = $self->_empty;
@@ -341,7 +341,7 @@ sub string {
 	my $last = pop(@line);					# last or only line
 	$last = join $tab, @core, "@rdata" unless scalar(@line);
 
-	return join "\n\t", @line, _wrap( $last, map "; $_", $self->_annotation );
+	return join "\n\t", @line, _wrap( $last, map {"; $_"} $self->_annotation );
 }
 
 
@@ -349,14 +349,14 @@ sub string {
 
     $plain = $rr->plain;
 
-Returns a simplified single line representation of the RR using the
-zone file format defined in RFC1035.  This facilitates interaction
-with programs like nsupdate which have rudimentary RR parsers.
+Returns a simplified single-line representation of the RR.
+This facilitates interaction with programs like nsupdate
+which have rudimentary parsers.
 
 =cut
 
 sub plain {
-	join ' ', shift->token;
+	return join ' ', shift->token;
 }
 
 
@@ -371,14 +371,17 @@ Returns a token list representation of the RR zone file string.
 sub token {
 	my $self = shift;
 
-	my @ttl = grep defined, $self->{ttl};
+	my @ttl	 = grep {defined} $self->{ttl};
 	my @core = ( $self->{owner}->string, @ttl, $self->class, $self->type );
 
-	my @rdata = $self->_empty ? () : eval { $self->_format_rdata };
-
 	# parse into quoted strings, contiguous non-whitespace and (discarded) comments
-	my @parse = map { s/\\\\/\\092/g; s/\\"/\\034/g; split /$PARSE_REGEX/o; } @rdata;
-	my @token = ( @core, grep defined && length, @parse );
+	local $_ = $self->_empty ? '' : join( ' ', $self->_format_rdata );
+	s/\\\\/\\092/g;						# disguise escaped escape
+	s/\\"/\\034/g;						# disguise escaped quote
+	s/\\\(/\\040/g;						# disguise escaped bracket
+	s/\\\)/\\041/g;						# disguise escaped bracket
+	s/\\;/\\059/g;						# disguise escaped semicolon
+	return ( @core, grep { defined && length } split /$PARSE_REGEX/o );
 }
 
 
@@ -395,14 +398,14 @@ and provisioning software.
 sub generic {
 	my $self = shift;
 
-	my @ttl = grep defined, $self->{ttl};
-	my @class = map "CLASS$_", grep defined, $self->{class};
-	my @core = ( $self->{owner}->string, @ttl, @class, "TYPE$self->{type}" );
-	my $data = $self->rdata;
-	my @data = ( '\\#', length($data), split /(\S{32})/, unpack 'H*', $data );
-	my @line = _wrap( "@core (", @data, ')' );
+	my @ttl	  = grep {defined} $self->{ttl};
+	my @class = map	 {"CLASS$_"} grep {defined} $self->{class};
+	my @core  = ( $self->{owner}->string, @ttl, @class, "TYPE$self->{type}" );
+	my $data  = $self->rdata;
+	my @data  = ( '\\#', length($data), split /(\S{32})/, unpack 'H*', $data );
+	my @line  = _wrap( "@core (", @data, ')' );
 	return join "\n\t", @line if scalar(@line) > 1;
-	join ' ', @core, @data;
+	return join ' ', @core, @data;
 }
 
 
@@ -416,11 +419,11 @@ Returns the owner name of the record.
 
 sub owner {
 	my $self = shift;
-	$self->{owner} = new Net::DNS::DomainName1035(shift) if scalar @_;
-	$self->{owner}->name if defined wantarray;
+	$self->{owner} = Net::DNS::DomainName1035->new(shift) if scalar @_;
+	return defined wantarray ? $self->{owner}->name : undef;
 }
 
-sub name { &owner; }			## historical
+sub name { return &owner; }		## historical
 
 
 =head2 type
@@ -434,7 +437,7 @@ Returns the record type.
 sub type {
 	my $self = shift;
 	croak 'not possible to change RR->type' if scalar @_;
-	typebyval( $self->{type} );
+	return typebyval( $self->{type} );
 }
 
 
@@ -449,7 +452,7 @@ Resource record class.
 sub class {
 	my $self = shift;
 	return $self->{class} = classbyname(shift) if scalar @_;
-	defined $self->{class} ? classbyval( $self->{class} ) : 'IN';
+	return defined $self->{class} ? classbyval( $self->{class} ) : 'IN';
 }
 
 
@@ -472,13 +475,13 @@ sub ttl {
 
 	return $self->{ttl} || 0 unless defined $time;		# avoid defining rr->{ttl}
 
-	my $ttl = 0;
+	my $ttl	 = 0;
 	my %time = reverse split /(\D)\D*/, $time . 'S';
 	while ( my ( $u, $t ) = each %time ) {
 		my $scale = $unit{uc $u} || die qq(bad time: $t$u);
 		$ttl += $t * $scale;
 	}
-	$self->{ttl} = $ttl;
+	return $self->{ttl} = $ttl;
 }
 
 
@@ -490,19 +493,18 @@ sub ttl {
 
 sub _decode_rdata {			## decode rdata from wire-format octet string
 	my ( $self, $data, $offset ) = @_;
-	$self->{rdata} = substr $$data, $offset, $self->{rdlength};
+	return $self->{rdata} = substr $$data, $offset, $self->{rdlength};
 }
 
 
 sub _encode_rdata {			## encode rdata as wire-format octet string
-	my $rdata = shift->{rdata};
+	return shift->{rdata};
 }
 
 
 sub _format_rdata {			## format rdata portion of RR string
-	my $data = shift->rdata;
-	my $size = length($data);				# RFC3597 unknown RR format
-	my @data = ( '\\#', $size, split /(\S{32})/, unpack 'H*', $data );
+	my $rdata = shift->rdata;				# RFC3597 unknown RR format
+	return ( '\\#', length($rdata), split /(\S{32})/, unpack 'H*', $rdata );
 }
 
 
@@ -513,6 +515,9 @@ sub _parse_rdata {			## parse RR attributes in argument list
 }
 
 
+sub _post_parse { }			## parser post processing
+
+
 sub _defaults { }			## set attribute default values
 
 
@@ -520,17 +525,17 @@ sub dump {				## print internal data structure
 	require Data::Dumper;					# uncoverable pod
 	local $Data::Dumper::Maxdepth = $Data::Dumper::Maxdepth || 6;
 	local $Data::Dumper::Sortkeys = $Data::Dumper::Sortkeys || 1;
-	print Data::Dumper::Dumper(@_);
+	return print Data::Dumper::Dumper(@_);
 }
 
 sub rdatastr {				## historical RR subtype method
-	&rdstring;						# uncoverable pod
+	return &rdstring;					# uncoverable pod
 }
 
 
 =head2 rdata
 
-    $rr = new Net::DNS::RR( type => NULL, rdata => 'arbitrary' );
+    $rr = Net::DNS::RR->new( type => NULL, rdata => 'arbitrary' );
 
 Resource record data section when viewed as opaque octets.
 
@@ -544,7 +549,8 @@ sub rdata {
 	my $data = shift || '';
 	my $hash = {};
 	$self->_decode_rdata( \$data, 0, $hash ) if ( $self->{rdlength} = length $data );
-	croak 'unexpected compression pointer in rdata' if keys %$hash;
+	croak 'compression pointer in rdata'	 if keys %$hash;
+	return;
 }
 
 
@@ -562,7 +568,7 @@ sub rdstring {
 	my @rdata = $self->_empty ? () : eval { $self->_format_rdata };
 	carp $@ if $@;
 
-	join "\n\t", _wrap(@rdata);
+	return join "\n\t", _wrap(@rdata);
 }
 
 
@@ -575,7 +581,7 @@ Returns the uncompressed length of the encoded RR-specific data.
 =cut
 
 sub rdlength {
-	length shift->rdata;
+	return length shift->rdata;
 }
 
 
@@ -622,6 +628,7 @@ sub set_rrsort_func {
 
 	my ($type) = $class =~ m/::([^:]+)$/;
 	$rrsortfunct{$type}{$attribute} = $function;
+	return;
 }
 
 
@@ -634,15 +641,15 @@ get_rrsort_func() returns a reference to the comparator function.
 
 =cut
 
-my $default = sub { $Net::DNS::a->canonical() cmp $Net::DNS::b->canonical(); };
+my $default = sub { return $Net::DNS::a->canonical() cmp $Net::DNS::b->canonical(); };
 
 sub get_rrsort_func {
-	my $class = shift;
+	my $class     = shift;
 	my $attribute = shift || 'default_sort';
 
 	my ($type) = $class =~ m/::([^:]+)$/;
 
-	$rrsortfunct{$type}{$attribute} || $default;
+	return $rrsortfunct{$type}{$attribute} || return $default;
 }
 
 
@@ -660,7 +667,7 @@ sub get_rrsort_func {
 # to be copied into the newly created object.
 
 our %_MINIMAL = ( 255 => bless ['type' => 255], __PACKAGE__ );
-our %_LOADED = %_MINIMAL;
+our %_LOADED  = %_MINIMAL;
 
 sub _subclass {
 	my ( $class, $rrname, $default ) = @_;
@@ -676,13 +683,13 @@ sub _subclass {
 
 			my $subclass = join '::', __PACKAGE__, $identifier;
 
-			unless ( eval "require $subclass" ) {
+			unless ( eval "require $subclass" ) {	## no critic ProhibitStringyEval
 				push @INC, sub {
 					Net::DNS::Parameters::_typespec("$rrtype.RRTYPE");
 				};
 
 				$subclass = join '::', __PACKAGE__, "TYPE$rrtype";
-				eval "require $subclass";
+				eval "require $subclass";	## no critic ProhibitStringyEval
 			}
 
 			$subclass = __PACKAGE__ if $@;
@@ -701,21 +708,31 @@ sub _subclass {
 	}
 
 	my $prebuilt = $default ? $_LOADED{$rrname} : $_MINIMAL{$rrname};
-	bless {@$prebuilt}, ref($prebuilt);			# create object
+	return bless {@$prebuilt}, ref($prebuilt);		# create object
 }
 
 
 sub _annotation {
 	my $self = shift;
 	$self->{annotation} = ["@_"] if scalar @_;
-	return @{$self->{annotation} || []} if wantarray;
+	return wantarray ? @{$self->{annotation} || []} : ();
 }
 
 
-my %ignore = map( ( $_ => 1 ), @core, 'annotation', '#' );
+my %warned;
+
+sub _deprecate {
+	my $msg = pop(@_);
+	carp join ' ', 'deprecated method;', $msg unless $warned{$msg}++;
+	return;
+}
+
+
+my %ignore = map { ( $_ => 1 ) } @core, 'annotation', '#';
 
 sub _empty {
-	not( $_[0]->{'#'} ||= scalar grep !$ignore{$_}, keys %{$_[0]} );
+	my $self = shift;
+	return not( $self->{'#'} ||= scalar grep { !$ignore{$_} } keys %$self );
 }
 
 
@@ -726,13 +743,16 @@ sub _wrap {
 
 	my ( @line, @fill );
 	foreach (@text) {
-		if ( ( $coln += 1 + length ) > $cols ) {	# start new line
-			push @line, join ' ', @fill if scalar @fill;
+		s/\\034/\\"/g;					# tart up escaped "
+		s/\\092/\\\\/g;					# tart up escaped escape
+		$coln += ( length || next ) + 1;
+		if ( $coln > $cols ) {				# start new line
+			push( @line, join ' ', @fill ) if @fill;
 			$coln = length;
 			@fill = ();
 		}
-		$coln = $cols if chomp;				# force line break
-		push( @fill, $_ );
+		$coln = $cols	  if chomp;			# force line break
+		push( @fill, $_ ) if length;
 	}
 	push @line, join ' ', @fill;
 	return @line;
@@ -746,21 +766,25 @@ our $AUTOLOAD;
 sub DESTROY { }				## Avoid tickling AUTOLOAD (in cleanup)
 
 sub AUTOLOAD {				## Default method
-	my $self = shift;
-	my $oref = ref($self);
-
-	no strict q/refs/;
+	my $self     = shift;
 	my ($method) = reverse split /::/, $AUTOLOAD;
+
+	for ($method) {			## tolerate mixed-case attribute name
+		return $self->$_(@_) if tr [A-Z-] [a-z_];
+	}
+
+	no strict 'refs';		## no critic ProhibitNoStrict
 	*{$AUTOLOAD} = sub {undef};	## suppress repetition and deep recursion
-	croak "$self has no class method '$method'" unless $oref;
+	my $oref = ref($self);
+	croak qq[$self has no class method "$method"] unless $oref;
 
 	my $string = $self->string;
-	my @object = grep defined($_), $oref, $oref->VERSION;
+	my @object = grep { defined($_) } $oref, $oref->VERSION;
 	my $module = join '::', __PACKAGE__, $self->type;
-	eval("require $module") if $oref eq __PACKAGE__;
+	eval("require $module") if $oref eq __PACKAGE__;	## no critic ProhibitStringyEval
 
-	@_ = ( <<"END", $@, "@object" );
-***  FATAL PROGRAM ERROR!!	Unknown instance method '$method'
+	@_ = ( <<"END" );
+***  FATAL PROGRAM ERROR!!	Unknown instance method "$method"
 ***  which the program has attempted to call for the object:
 ***
 $string
@@ -769,6 +793,8 @@ $string
 ***  that the object would be of a particular type.  The type of an
 ***  object should be checked before calling any of its methods.
 ***
+@object
+$@
 END
 	goto &{'Carp::confess'};
 }
