@@ -1,22 +1,18 @@
 package Net::DNS::RR::AAAA;
 
-#
-# $Id: AAAA.pm 1382 2015-08-26 20:39:33Z willem $
-#
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1382 $)[1];
-
-
 use strict;
+use warnings;
+our $VERSION = (qw$Id: AAAA.pm 1814 2020-10-14 21:49:16Z willem $)[2];
+
 use base qw(Net::DNS::RR);
 
+
 =head1 NAME
 
 Net::DNS::RR::AAAA - DNS AAAA resource record
 
 =cut
 
-
 use integer;
 
 
@@ -25,21 +21,20 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 	my ( $data, $offset ) = @_;
 
 	$self->{address} = unpack "\@$offset a16", $$data;
+	return;
 }
 
 
 sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	return '' unless $self->{address};
-	pack 'a16', $self->{address};
+	return pack 'a16', $self->{address};
 }
 
 
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless $self->{address};
 	return $self->address_short;
 }
 
@@ -48,32 +43,32 @@ sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
 	$self->address(shift);
+	return;
 }
 
 
-my $pad = pack 'x16';
-
 sub address_long {
-	sprintf '%x:%x:%x:%x:%x:%x:%x:%x', unpack 'n8', shift->{address} . $pad;
+	my $addr = pack 'a*@16', grep {defined} shift->{address};
+	return sprintf '%x:%x:%x:%x:%x:%x:%x:%x', unpack 'n8', $addr;
 }
 
 
 sub address_short {
-	for ( sprintf ':%x:%x:%x:%x:%x:%x:%x:%x:', unpack 'n8', shift->{address} . $pad ) {
-		s/(:0[:0]+:)(?!.+:0\1)/::/;			# squash longest zero sequence
-		s/^:// unless /^::/;				# prune LH :
-		s/:$// unless /::$/;				# prune RH :
-		return $_;
-	}
+	my $addr = pack 'a*@16', grep {defined} shift->{address};
+	local $_ = sprintf ':%x:%x:%x:%x:%x:%x:%x:%x:', unpack 'n8', $addr;
+	s/(:0[:0]+:)(?!.+:0\1)/::/;				# squash longest zero sequence
+	s/^:// unless /^::/;					# prune LH :
+	s/:$// unless /::$/;					# prune RH :
+	return $_;
 }
 
 
 sub address {
 	my $self = shift;
-	my $addr = shift;
 
-	return $self->address_long unless defined $addr;
+	return address_long($self) unless scalar @_;
 
+	my $addr  = shift;
 	my @parse = split /:/, "0$addr";
 
 	if ( (@parse)[$#parse] =~ /\./ ) {			# embedded IPv4
@@ -85,7 +80,7 @@ sub address {
 
 	# Note: pack() masks overlarge values, mostly without warning.
 	my @expand = map { /./ ? hex($_) : (0) x ( 9 - @parse ) } @parse;
-	$self->{address} = pack 'n8', @expand;
+	return $self->{address} = pack 'n8', @expand;
 }
 
 
@@ -96,9 +91,9 @@ __END__
 =head1 SYNOPSIS
 
     use Net::DNS;
-    $rr = new Net::DNS::RR('name IN AAAA address');
+    $rr = Net::DNS::RR->new('name IN AAAA address');
 
-    $rr = new Net::DNS::RR(
+    $rr = Net::DNS::RR->new(
 	name	=> 'example.com',
 	type	=> 'AAAA',
 	address => '2001:DB8::8:800:200C:417A'
