@@ -1,10 +1,9 @@
 package Net::DNS::Text;
 
-#
-# $Id: Text.pm 1360 2015-06-15 09:58:53Z willem $
-#
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1360 $)[1];
+use strict;
+use warnings;
+
+our $VERSION = (qw$Id: Text.pm 1847 2021-08-11 10:02:44Z willem $)[2];
 
 
 =head1 NAME
@@ -15,11 +14,11 @@ Net::DNS::Text - DNS text representation
 
     use Net::DNS::Text;
 
-    $object = new Net::DNS::Text('example');
+    $object = Net::DNS::Text->new('example');
     $string = $object->string;
 
-    $object = decode Net::DNS::Text( \$data, $offset );
-    ( $object, $next ) = decode Net::DNS::Text( \$data, $offset );
+    $object = Net::DNS::Text->decode( \$data, $offset );
+    ( $object, $next ) = Net::DNS::Text->decode( \$data, $offset );
 
     $data = $object->encode;
     $text = $object->value;
@@ -35,30 +34,25 @@ lifetime.
 =cut
 
 
-use strict;
 use integer;
 use Carp;
 
 
-use constant ASCII => ref( eval {
+use constant ASCII => ref eval {
 	require Encode;
-	Encode::find_encoding('ascii');				# encoding object
-} );
-
-use constant LIBUTF8 => eval {
-	Encode::decode_utf8( chr(91) ) eq '[';			# not UTF-EBCDIC  [see UTR#16 3.6]
+	Encode::find_encoding('ascii');
 };
 
-use constant UTF8 => ref( eval {
-	LIBUTF8 && Encode::find_encoding('utf8');		# encoding object
-} );
+use constant UTF8 => scalar eval {	## not UTF-EBCDIC  [see Unicode TR#16 3.6]
+	Encode::encode_utf8( chr(182) ) eq pack( 'H*', 'C2B6' );
+};
 
 
 =head1 METHODS
 
 =head2 new
 
-    $object = new Net::DNS::Text('example');
+    $object = Net::DNS::Text->new('example');
 
 Creates a text object which encapsulates a single character
 string component of a resource record.
@@ -72,7 +66,7 @@ interpretation.
 
 =cut
 
-my %unescape;				## precalculated numeric escape table
+my ( %escape, %escapeUTF8, %unescape );	## precalculated escape tables
 
 sub new {
 	my $self = bless [], shift;
@@ -88,7 +82,7 @@ sub new {
 
 	while ( length $_ > 255 ) {
 		my $chunk = substr( $_, 0, 255 );		# carve into chunks
-		substr( $chunk, -length($1) ) = '' if $chunk =~ /.([\300-\377][\200-\277]*)$/;
+		$chunk =~ s/[\300-\377][\200-\277]*$//;
 		push @$self, $chunk;
 		substr( $_, 0, length $chunk ) = '';
 	}
@@ -100,9 +94,9 @@ sub new {
 
 =head2 decode
 
-    $object = decode Net::DNS::Text( \$buffer, $offset );
+    $object = Net::DNS::Text->decode( \$buffer, $offset );
 
-    ( $object, $next ) = decode Net::DNS::Text( \$buffer, $offset );
+    ( $object, $next ) = Net::DNS::Text->decode( \$buffer, $offset );
 
 Creates a text object which represents the decoded data at the
 indicated offset within the data buffer.
@@ -146,15 +140,22 @@ suitable for inclusion in a DNS packet buffer.
 
 sub encode {
 	my $self = shift;
-	join '', map pack( 'C a*', length $_, $_ ), @$self;
+	return join '', map { pack( 'C a*', length $_, $_ ) } @$self;
 }
 
 
-# Returns unbounded wire-format text
+=head2 raw
+
+    $data = $object->raw;
+
+Returns the wire-format encoded representation of the text object
+without the explicit length field.
+
+=cut
 
 sub raw {
 	my $self = shift;
-	join '', map pack( 'a*', $_ ), @$self;
+	return join '', map { pack( 'a*', $_ ) } @$self;
 }
 
 
@@ -167,8 +168,9 @@ Character string representation of the text object.
 =cut
 
 sub value {
+	return unless defined wantarray;
 	my $self = shift;
-	_decode_utf8( join '', @$self );
+	return _decode_utf8( join '', @$self );
 }
 
 
@@ -176,22 +178,33 @@ sub value {
 
     $string = $text->string;
 
-Conditionally quoted zone file representation of the text object.
+Conditionally quoted RFC1035 zone file representation of the text object.
 
 =cut
 
-my %escape;				## precalculated ASCII/UTF-8 escape table
-
 sub string {
 	my $self = shift;
 
-	my @s = map split( '', $_ ), @$self;			# escape non-printable
-	my $string = _decode_utf8( join '', map $escape{$_}, @s );
+	my @s = map { split '', $_ } @$self;			# escape special and ASCII non-printable
+	my $s = _decode_utf8( join '', map { $escape{$_} } @s );
+	return $s =~ /[ \t\n\r\f(),;]|^$/ ? qq("$s") : $s;	# quote special characters and empty string
+}
 
-	return $string unless $string =~ /^$|[ \t\n\r\f]/;	# unquoted contiguous
 
-	$string =~ s/\\([$();@])/$1/g;				# nothing special within quotes
-	join '', '"', $string, '"';				# quoted string
+=head2 unicode
+
+    $string = $text->unicode;
+
+Conditionally quoted Unicode representation of the text object.
+
+=cut
+
+sub unicode {
+	my $self = shift;
+
+	my @s = map { split '', $_ } @$self;			# escape special and non-printable
+	my $s = _decode_utf8( join '', map { $escapeUTF8{$_} } @s );
+	return $s =~ /[ \t\n\r\f();]|^$/ ? qq("$s") : $s;	# quote special characters and empty string
 }
 
 
@@ -202,69 +215,52 @@ my $ascii = ASCII ? Encode::find_encoding('ascii') : undef;	# Osborn's Law:
 my $utf8  = UTF8  ? Encode::find_encoding('utf8')  : undef;	# Variables won't; constants aren't.
 
 
-my $decode_ascii = sub {		## ASCII to perl internal encoding
-	my $s = shift;
+sub _decode_utf8 {			## UTF-8 to perl internal encoding
+	local $_ = shift;
 
 	# partial transliteration for non-ASCII character encodings
-	$s =~ tr
+	tr
 	[\040-\176\000-\377]
-	[ !"#$%&'()*+,-./0-9:;<=>?@A-Z\[\\\]^_`a-z{|}~?] unless ASCII;
+	[ !"#$%&'()*+,\-./0-9:;<=>?@A-Z\[\\\]^_`a-z{|}~?] unless ASCII;
 
-	my $z = length substr $s, 0, 0;				# pre-5.18 taint workaround
-	ASCII ? pack( "a* x$z", $ascii->decode($s) ) : $s;
-};
-
-sub _decode_utf8 {			## UTF-8 to perl internal encoding
-	my $s = shift;
-	UTF8 ? ( $utf8->decode($s) . substr $s, 0, 0 ) : &$decode_ascii($s);
+	my $z = length($_) - length($_);			# pre-5.18 taint workaround
+	return ASCII ? substr( ( UTF8 ? $utf8 : $ascii )->decode($_), $z ) : $_;
 }
 
 
-my $encode_ascii = sub {		## perl internal encoding to ASCII
-	my $s = shift;
-
-	# partial transliteration for non-ASCII character encodings
-	$s =~ tr
-	[ !"#$%&'()*+,-./0-9:;<=>?@A-Z\[\\\]^_`a-z{|}~]
-	[\040-\176] unless ASCII;
-
-	my $z = length substr $s, 0, 0;				# pre-5.18 taint workaround
-	ASCII ? pack( "a* x$z", $ascii->encode($s) ) : $s;
-};
-
 sub _encode_utf8 {			## perl internal encoding to UTF-8
-	my $s = shift;
+	local $_ = shift;
 
-	my $z = length substr $s, 0, 0;				# pre-5.18 taint workaround
-	UTF8 ? pack( "a* x$z", $utf8->encode($s) ) : &$encode_ascii($s);
+	# partial transliteration for non-ASCII character encodings
+	tr
+	[ !"#$%&'()*+,\-./0-9:;<=>?@A-Z\[\\\]^_`a-z{|}~]
+	[\040-\176] unless ASCII;
+
+	my $z = length($_) - length($_);			# pre-5.18 taint workaround
+	return ASCII ? substr( ( UTF8 ? $utf8 : $ascii )->encode($_), $z ) : $_;
 }
 
 
-%escape = eval {			## precalculated ASCII/UTF-8 escape table
-	my %table;
-	my @C0 = ( 0 .. 31 );					# control characters
-	my @NA = UTF8 ? ( 192, 193, 216 .. 223, 245 .. 255 ) : ( 128 .. 255 );
+%escape = eval {			## precalculated ASCII escape table
+	my %table = map { ( chr($_) => chr($_) ) } ( 0 .. 127 );
 
-	foreach ( 0 .. 255 ) {					# transparent
-		$table{pack( 'C', $_ )} = pack 'C', $_;
-	}
-
-	foreach ( 34, 36, 40, 41, 59, 64, 92 ) {		# escape character
-		$table{pack( 'C', $_ )} = pack 'C2', 92, $_;
-	}
-
-	foreach my $n ( @C0, 127, @NA ) {			# \ddd
+	foreach my $n ( 0 .. 31, 34, 92, 127 .. 255 ) {		# numerical escape
 		my $codepoint = sprintf( '%03u', $n );
 
-		# partial transliteration for non-ASCII character encodings
+		# transliteration for non-ASCII character encodings
 		$codepoint =~ tr [0-9] [\060-\071];
 
-		$table{pack( 'C', $n )} = pack 'C a3', 92, $codepoint;
+		$table{chr($n)} = pack 'C a3', 92, $codepoint;
 	}
 
 	return %table;
 };
 
+%escapeUTF8 = eval {			## precalculated UTF-8 escape table
+	my @octet = UTF8 ? ( 128 .. 191, 194 .. 254 ) : ();
+	return ( %escape, map { ( chr($_) => chr($_) ) } @octet );
+};
+
 
 %unescape = eval {			## precalculated numeric escape table
 	my %table;
@@ -272,12 +268,12 @@ sub _encode_utf8 {			## perl internal encoding to UTF-8
 	foreach my $n ( 0 .. 255 ) {
 		my $key = sprintf( '%03u', $n );
 
-		# partial transliteration for non-ASCII character encodings
+		# transliteration for non-ASCII character encodings
 		$key =~ tr [0-9] [\060-\071];
 
 		$table{$key} = pack 'C', $n;
-		$table{$key} = pack 'C2', 92, $n if $n == 92;	   # escaped escape
 	}
+	$table{"\060\071\062"} = pack 'C2', 92, 92;		# escaped escape
 
 	return %table;
 };
@@ -327,8 +323,7 @@ DEALINGS IN THE SOFTWARE.
 
 =head1 SEE ALSO
 
-L<perl>, L<Net::DNS>, RFC1035, RFC3629,
-Unicode Technical Report #16
+L<perl>, L<Net::DNS>, RFC1035, RFC3629, Unicode TR#16
 
 =cut
 
